#pragma once

#include <vector>

#include "glm/glm.hpp"

typedef std::vector<double> Vector6;

class Orbit
{
public:
    Orbit();
    ~Orbit() = default; 

    double getSemiMajorAxis() const;
    double getEccentricity() const;
    double getInclination() const;
    double getArgumentOfPeriapsis() const;
    double getLongitudeOfAscendingNode() const;

    void setSemiMajorAxis(double semiMajorAxis);
    void setEccentricity(double eccentricity);
    void setInclination(double inclination);
    void setArgumentOfPeriapsis(double argumentOfPeriapsis);
    void setLongitudeOfAscendingNode(double longitudeOfAscendingNode);

    // TODO: meanAnomaly in all these arguments actually means ellipticalMeanAnomaly,
    // will have to change that when adding non-ellipctical orbits - don't get confused!
    //
    // do we actual want to use mean anomaly for this? we could instead pass in time and
    // the gravitational parameter which are easier numbers to come by, i think? at least,
    // the mean motion is determined from the mean motion and time time
    //glm::vec3 getPosition(double gravitationalParameter, double time) const;

    glm::dvec3 getPosition(const double meanAnomaly) const;
    glm::dvec3 getTangent(const double meanAnomaly) const;
    //glm::mat4 getLookAlongMatrix(const double meanAnomaly) const;

private:
    Vector6 _keplerianElements;

    const double getEccentricAnomaly(const double meanAnomaly) const;
    const double getTrueAnomaly(const double meanAnomaly) const;
    const Vector6 getCartesianCoordinates(const double meanAnomaly) const;
};
